local pl_stringx = require("pl.stringx")
local json = require("json")
local utils = require("utils2")
local shadow = require("shadow")

utils.check_auth()
utils.check_permission("DeviceManage")

local function current_time()
    ngx.update_time()
    return ngx.now() * 1000
end

local function get_properties(product, device, identifiers)
    local thing_shadow = shadow:new(nil, product, device)
    local ok, properties, err = thing_shadow:get_properties(identifiers)
    if not ok then
        utils.error_exit(ngx.HTTP_INTERNAL_SERVER_ERROR, err)
    end

    local resp = {}
    resp["Code"] = 200
    resp["Message"] = "success"
    resp["Data"] = {}
    resp["Data"]["Properties"] = properties
    resp["Data"]["Timestamp"] = current_time()
    local str = json.encode(resp)

    ngx.header['Content-Type'] = 'application/json; charset=utf-8'
    ngx.say(str)
end

local function set_properties(product, device, properties)
    local thing_shadow = shadow:new(nil, product, device)
    local ok, res, err = thing_shadow:set_properties(properties)
    if not ok then
        utils.error_exit(ngx.HTTP_INTERNAL_SERVER_ERROR, err)
    end

    local resp = {}
    resp["Code"] = 200
    resp["Message"] = "success"
    resp["Data"] = {}
    resp["Data"]["Data"] = res
    resp["Data"]["Timestamp"] = current_time()
    local str = json.encode(resp)

    ngx.header['Content-Type'] = 'application/json; charset=utf-8'
    ngx.say(str)
end

local product = ngx.var[1]
utils.check_pk(product)

local device = ngx.var[2]
utils.check_dn(device)

local request_method = ngx.var.request_method
if request_method == "GET" then
    -- Both query arguments forms are supported:
    -- 1) identifiers=identifier1,identifier2,...
    -- 2) identifiers=identifier1&identifiers&identifier2&...
    local args = ngx.req.get_uri_args()
    local identifiers = args["identifiers"]
    if type(identifiers) == "string" then
        if #identifiers == 0 then
            return utils.error_exit(ngx.HTTP_BAD_REQUEST, "identifiers is empty")
        end
        identifiers = pl_stringx.split(identifiers, ",")
    end
    if type(identifiers) ~= "table" or next(identifiers) == nil then
        return utils.error_exit(ngx.HTTP_BAD_REQUEST, "identifiers not found or is empty")
    end
    return get_properties(product, device, identifiers)
elseif request_method == "POST" then
    ngx.req.read_body()
    local body = ngx.req.get_body_data()
    if not body then
        return utils.error_exit(ngx.HTTP_BAD_REQUEST, "no body found")
    end

    local ok, args = pcall(json.decode, body)
    if not ok then
        return utils.error_exit(ngx.HTTP_BAD_REQUEST, "not a JSON body")
    end

    local properties = args["Properties"]
    if type(properties) ~= "table" then
        return utils.error_exit(ngx.HTTP_BAD_REQUEST, "Properties not a object or is empty")
    end
    local k = next(properties)
    if not k then
        return utils.error_exit(ngx.HTTP_BAD_REQUEST, "Properties not a object or is empty")
    end

    return set_properties(product, device, properties)
else
    return utils.error_exit(ngx.HTTP_NOT_ALLOWED, "request method not allowed: " .. request_method)
end